
package java2D2compositing;
import java.awt.AlphaComposite;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
//import java.awt.geom.*;
import java.awt.image.BufferedImage;
import java.util.Iterator;
import java.util.List;

/**
 * Komponente, auf der gezeichnet wird
 * @author arnd
 */
public class RenderCanvas extends Canvas {
    
    private RenderingHints qualityHints;
    private Graphics2D graphics2D;
    private ObjectRenderer objectRenderer;
    private CanvasModel cModel;
    
    
    RenderCanvas(CanvasModel cm) {
        cModel = cm;
        setBackground(Color.black);
        this.objectRenderer = new ObjectRenderer();
        this.qualityHints = new RenderingHints(null);
        
        
        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                
                int x = e.getX();
                int y = e.getY();
                
                AffineTransform pos = AffineTransform.getTranslateInstance(x,y);
                pos.rotate(State.getAngle()*Math.PI/180, 75, 75);
                GeometricObject go = new GeometricObject(State.getShapeOption(),State.getFillOption(),  pos);
                

                
                cModel.addObject(go);
                
                if(State.isAntialiasingEnabled()) {
                    qualityHints.put(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
                } else {
                    qualityHints.put(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);
                }
                if(State.isDitheringEnabled()) {
                    qualityHints.put(RenderingHints.KEY_DITHERING,RenderingHints.VALUE_DITHER_ENABLE);
                } else {
                    qualityHints.put(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_DISABLE);
                }
                repaint();
                
            }
        });
    }
    
    public void paint(Graphics g) {
        
        //Cast Graphics g to Graphics2D graphics2D
        graphics2D = (Graphics2D) g;
        
        //RenderingHints setzen
        graphics2D.setRenderingHints(qualityHints);
        
        Dimension d = getSize();
        int w = d.width;
        int h = d.height;
        
        // Creates the buffered image.
        BufferedImage buffImg = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
        Graphics2D gbi = buffImg.createGraphics();
        
        
        List objects = cModel.getObjects();
        
        //Erstes Objekt immer "normal" rendern
        AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER);
        gbi.setComposite(ac);
        
        final Iterator it = objects.iterator();
        
        
        //Erstes Objekt immer "normal" rendern
        if (it.hasNext()) {
            GeometricObject geometricObject = (GeometricObject)it.next();
            this.objectRenderer.renderObject(gbi, geometricObject);
        }
        
        //Andere Objekte mit eingestellter Compositing Rule rendern
        ac = AlphaComposite.getInstance(State.getCompositingSelection());
        
        gbi.setComposite(ac);
        while (it.hasNext()) {
            GeometricObject geometricObject = (GeometricObject)it.next();
            this.objectRenderer.renderObject(gbi, geometricObject);
        }
        
        
        
        graphics2D.drawImage(buffImg, null, 0, 0);
    }
    
    public Dimension getPreferredSize() {
        return new Dimension(400,400);
    }
}
